/**
 * @file
 * Provides Intersection Observer for ajaxified blocks and Views pager.
 */

(function (Drupal) {

  'use strict';

  Drupal.blazy = Drupal.blazy || {};
  Drupal.io = Drupal.io || {};
  var _errorTimer;

  /**
   * Intersection Observer API public methods.
   *
   * @namespace
   */
  Drupal.io.base = Drupal.io.base || {
    init: null,
    ajaxin: false,
    initAjaxin: function (settings) {
      return 'Ajaxin' in window ? new Ajaxin(settings) : false;
    },

    isError: function (el) {
      return el !== null && (el.getAttribute(this.dataHit) === '-1' || el.classList.contains(this.settings.errorClass));
    },

    error: function (el) {
      var me = this;

      // This basically means, re-using bLazy to watch the intersected elements.
      el.classList.add(me.settings.errorClass);
      el.setAttribute(me.dataHit, -1);

      me.reIntersecting(el);
    },

    success: function (el, status, parent) {
      var me = this;
      parent = parent || el.parentNode;

      // Marks it done.
      el.classList.add(me.settings.successClass);
      el.setAttribute(me.dataHit, status);

      // To inform themers if anything to do with the loaded contents.
      if (parent !== null) {
        parent.setAttribute(me.dataLoaded, status);
      }
    },

    reIntersecting: function (el) {
      var me = this;

      // Attempts to re-ajax if its content is not loaded, prevents infinite.
      if (el.getAttribute(me.dataHit) === '1') {
        return;
      }

      // Mark it an explicit error, so to re-request.
      el.setAttribute(me.dataHit, -1);

      // DOM ready fix. This is mostly needed by the fallback bLazy.
      // Or if IO disconnect option is enabled.
      window.clearTimeout(_errorTimer);
      _errorTimer = window.setTimeout(function () {
        me.intersecting(el);
      }, 300);
    },

    isIo: function () {
      return Drupal.blazy.isIo();
    },

    run: function (opts) {
      return this.isIo() ? new Bio(opts) : new Blazy(opts);
    },

    // @todo remove for dBlazy.context post Blazy:2.6+.
    context: function (context) {
      // Weirdo: context may be null after Colorbox close.
      context = context || document;

      // jQuery may pass its object as non-expected context identified by length.
      context = 'length' in context ? context[0] : context;
      return context instanceof HTMLDocument ? context : document;
    }
  };

})(Drupal);
